from .base_classes import Container


class ContactAddress(Container):
    """The HTML `<address>` element indicates that the enclosed HTML
    provides contact information for a person or people, or for an
    organisation.
    """

    __slots__ = ()
    tag = "address"


Address = ContactAddress


class ArticleContents(Container):
    """The HTML `<article>` element represents a self-contained
    composition in a document, page, application, or site, which is
    intended to be independently distributable or reusable (e.g.,
    in syndication).
    """

    __slots__ = ()
    tag = "article"


Article = ArticleContents


class Aside(Container):
    """The HTML `<aside>` element represents a portion of a document
    whose content is only indirectly related to the document's main
    content.
    """

    __slots__ = ()
    tag = "aside"


class Footer(Container):
    """The HTML `<footer>` element represents a footer for its nearest
    sectioning content or sectioning root element. A footer typically
    contains information about the author of the section, copyright
    data or links to related documents.
    """

    __slots__ = ()
    tag = "footer"


class Header(Container):
    """The HTML `<header>` element represents introductory content,
    typically a group of introductory or navigational aids. It may
    contain some heading elements but also a logo, a search form, an
    author name, and other elements.
    """

    __slots__ = ()
    tag = "header"


class Heading1(Container):
    """The HTML `<h1>`–`<h6>` elements represent six levels of section
    headings. `<h1>` is the highest section level and `<h6>` is the
    lowest.
    """

    __slots__ = ()
    tag = "h1"


class Heading2(Container):
    """The HTML `<h1>`–`<h6>` elements represent six levels of section
    headings. `<h1>` is the highest section level and `<h6>` is the
    lowest.
    """

    __slots__ = ()
    tag = "h2"


class Heading3(Container):
    """The HTML `<h1>`–`<h6>` elements represent six levels of section
    headings. `<h1>` is the highest section level and `<h6>` is the
    lowest.
    """

    __slots__ = ()
    tag = "h3"


class Heading4(Container):
    """The HTML `<h1>`–`<h6>` elements represent six levels of section
    headings. `<h1>` is the highest section level and `<h6>` is the
    lowest.
    """

    __slots__ = ()
    tag = "h4"


class Heading5(Container):
    """The HTML `<h1>`–`<h6>` elements represent six levels of section
    headings. `<h1>` is the highest section level and `<h6>` is the
    lowest.
    """

    __slots__ = ()
    tag = "h5"


class Heading6(Container):
    """The HTML `<h1>`–`<h6>` elements represent six levels of section
    headings. `<h1>` is the highest section level and `<h6>` is the
    lowest.
    """

    __slots__ = ()
    tag = "h6"


H1 = Heading1
H2 = Heading2
H3 = Heading3
H4 = Heading4
H5 = Heading5
H6 = Heading6


class HeadingGroup(Container):
    """The HTML `<hgroup>` element represents a multi-level heading for
    a section of a document. It groups a set of `<h1>`–`<h6>` elements.
    """

    __slots__ = ()
    tag = "hgroup"


HGroup = HeadingGroup


class MainContent(Container):
    """The HTML `<main>` element represents the dominant content of the
    `<body>` of a document. The main content area consists of content
    that is directly related to or expands upon the central topic of a
    document, or the central functionality of an application.
    """

    __slots__ = ()
    tag = "main"


Main = MainContent


class Navigation(Container):
    """The HTML `<nav>` element represents a section of a page whose
    purpose is to provide navigation links, either within the current
    document or to other documents. Common examples of navigation
    sections are menus, tables of contents, and indexes.
    """

    __slots__ = ()
    tag = "nav"


Nav = Navigation


class Section(Container):
    """The HTML `<section>` element represents a standalone section -
    which doesn't have a more specific semantic element to represent it
    - contained within an HTML document.
    """

    __slots__ = ()
    tag = "section"


class HTML(Container):
    """The HTML `<html>` element represents the root (top-level element)
    of an HTML document, so it is also referred to as the root element.
    All other elements must be descendants of this element.
    """

    __slots__ = ()
    tag = "html"

    def serialise(self, minify: bool = True) -> str:
        return (
            "<!DOCTYPE html>"
            + ("" if minify else "\n")
            + super().serialise(minify=minify)
        )


class Head(Container):
    """The HTML `<head>` element contains machine-readable information
    (metadata) about the document, like its title, scripts, and style
    sheets.
    """

    __slots__ = ()
    tag = "head"


class Body(Container):
    """The HTML `<body>` Element represents the content of an HTML document.
    There can be only one `<body>` element in a document.
    """

    __slots__ = ()
    tag = "body"



import typing

from .base_classes import Container, Void


class TableCaption(Container):
    """The HTML `<caption>` element specifies the caption (or title) of a
    table.
    """

    __slots__ = ()
    tag = "caption"


Caption = TableCaption


class ColumnDeclaration(Void):
    """The HTML `<col>` element defines a column within a table and is used
    for defining common semantics on all common cells. It is generally found
    within a `<colgroup>` element.

    ```html
    <table>
        <caption>Superheros and sidekicks</caption>
        <colgroup>
            <col>
            <col span="2" class="batman">
            <col span="2" class="flash">
        </colgroup>
        <tr>
            <td> </td>
            <th scope="col">Batman</th>
            <th scope="col">Robin</th>
            <th scope="col">The Flash</th>
            <th scope="col">Kid Flash</th>
        </tr>
        <tr>
            <th scope="row">Skill</th>
            <td>Smarts</td>
            <td>Dex, acrobat</td>
            <td>Super speed</td>
            <td>Super speed</td>
        </tr>
    </table>
    ```
    """

    __slots__ = ()
    tag = "col"


Col = ColumnDeclaration


class ColumnGroup(Container):
    """The HTML `<colgroup>` element defines a group of columns within a
    table.

    ```html
    <table>
        <caption>Superheros and sidekicks</caption>
        <colgroup>
            <col>
            <col span="2" class="batman">
            <col span="2" class="flash">
        </colgroup>
        <tr>
            <td> </td>
            <th scope="col">Batman</th>
            <th scope="col">Robin</th>
            <th scope="col">The Flash</th>
            <th scope="col">Kid Flash</th>
        </tr>
        <tr>
            <th scope="row">Skill</th>
            <td>Smarts</td>
            <td>Dex, acrobat</td>
            <td>Super speed</td>
            <td>Super speed</td>
        </tr>
    </table>
    ```
    """

    __slots__ = ()
    tag = "colgroup"


ColGroup = ColumnGroup


class TableDataCell(Container):
    """The HTML `<td>` element defines a cell of a table that contains data.
    It participates in the table model.
    """

    __slots__ = ()
    tag = "td"


TD = TableDataCell


class TableHeaderCell(Container):
    """The HTML `<th>` element defines a cell as header of a group of table
    cells. The exact nature of this group is defined by the scope and headers
    attributes.
    """

    __slots__ = ()
    tag = "th"


TH = TableHeaderCell


class TableRow(Container):
    """The HTML `<tr>` element defines a row of cells in a table. The row's
    cells can then be established using a mix of `<td>` (data cell) and `<th>`
    (header cell) elements.
    """

    __slots__ = ()
    tag = "tr"


TR = TableRow


class TableHead(Container):
    """The HTML `<thead>` element defines a set of rows defining the head of
    the columns of the table.
    """

    __slots__ = ()
    tag = "thead"


THead = TableHead


class TableBody(Container):
    """The HTML Table Body element (`<tbody>`) encapsulates a set of table
    rows (`<tr>` elements), indicating that they comprise the body of the
    table (`<table>`).
    """

    tag = "tbody"


TBody = TableBody


class TableFoot(Container):
    """The HTML `<tfoot>` element defines a set of rows summarising the
    columns of the table.
    """

    __slots__ = ()
    tag = "tfoot"


TFoot = TableFoot


class Table(Container):
    """The HTML `<table>` element represents tabular data — that is,
    information presented in a two-dimensional table comprised of rows and
    columns of cells containing data.
    """

    __slots__ = ()
    tag = "table"



import typing

from .base_classes import Container, Void


class Button(Container):
    """The HTML `<button>` element represents a clickable button, used to
    submit forms or anywhere in a document for accessible, standard button
    functionality.
    """

    __slots__ = ()
    tag = "button"


class DataList(Container):
    """The HTML `<datalist>` element contains a set of `<option>` elements
    that represent the permissible or recommended options available to choose
    from within other controls.

    ```html
    <label for="ice-cream-choice">Choose a flavour:</label>
    <input list="ice-cream-flavours" id="ice-cream-choice" name="ice-cream-choice" />

    <datalist id="ice-cream-flavours">
        <option value="Chocolate">
        <option value="Coconut">
        <option value="Mint">
        <option value="Strawberry">
        <option value="Vanilla">
    </datalist>
    ```
    """

    __slots__ = ()
    tag = "datalist"


class FieldSet(Container):
    """The HTML `<fieldset>` element is used to group several controls as well
    as labels (`<label>`) within a web form.

    ```html
    <form>
        <fieldset>
            <legend>Choose your favourite monster</legend>

            <input type="radio" id="kraken" name="monster">
            <label for="kraken">Kraken</label><br/>

            <input type="radio" id="sasquatch" name="monster">
            <label for="sasquatch">Sasquatch</label><br/>

            <input type="radio" id="mothman" name="monster">
            <label for="mothman">Mothman</label>
        </fieldset>
    </form>
    ```
    """

    __slots__ = ()
    tag = "fieldset"


class Form(Container):
    """The HTML `<form>` element represents a document section containing
    interactive controls for submitting information.
    """

    __slots__ = ()
    tag = "form"


class Input(Void):
    """The HTML `<input>` element is used to create interactive controls for
    web-based forms in order to accept data from the user; a wide variety of
    types of input data and control widgets are available, depending on the
    device and user agent.

    If used directly, `type` must be provided
    """

    __slots__ = ("type",)
    tag = "input"
    type: str

    def apply_attributes(self, attributes: typing.Dict[str, typing.Any]) -> None:
        if not attributes["type"]:
            attributes["type"] = self.type
        return super().apply_attributes(attributes)


class Checkbox(Input):
    """A check box allowing single values to be selected/deselected."""

    __slots__ = ()
    type = "checkbox"


class ColourInput(Input):
    """A control for specifying a colour; opening a colour picker when active
    in supporting browsers.
    """

    __slots__ = ()
    type = "color"


ColorInput = ColourInput
Color = ColourInput


class DateInput(Input):
    """A control for entering a date (year, month, and day, with no time).
    Opens a date picker or numeric wheels for year, month, day when active in
    supporting browsers.
    """

    __slots__ = ()
    type = "date"


Date = DateInput


class DatetimeInput(Input):
    """A control for entering a date and time, with no time zone. Opens a date
    picker or numeric wheels for date- and time-components when active in
    supporting browsers.
    """

    __slots__ = ()
    type = "datetime-local"


DatetimeLocal = DatetimeInput


class EmailInput(Input):
    """A field for editing an email address. Looks like a text input, but has
    validation parameters and relevant keyboard in supporting browsers and
    devices with dynamic keyboards.
    """

    __slots__ = ()
    type = "email"


Email = EmailInput


class FileInput(Input):
    """A control that lets the user select a file. Use the accept attribute to
    define the types of files that the control can select.
    """

    __slots__ = ()
    type = "file"


File = FileInput


class HiddenInput(Input):
    """A control that is not displayed but whose value is submitted to the
    server. There is an example in the next column, but it's hidden!
    """

    __slots__ = ()
    type = "hidden"


Hidden = HiddenInput


class ImageButton(Input):
    """A graphical submit button. Displays an image defined by the src
    attribute. The alt attribute displays if the image src is missing.
    """

    __slots__ = ()
    type = "image"


Image = ImageButton


class MonthInput(Input):
    """A control for entering a month and year, with no time zone."""

    __slots__ = ()
    type = "month"


Month = MonthInput


class NumberInput(Input):
    """A control for entering a number. Displays a spinner and adds default
    validation when supported. Displays a numeric keypad in some devices with
    dynamic keypads.
    """

    __slots__ = ()
    type = "number"


Number = NumberInput


class PasswordInput(Input):
    """A single-line text field whose value is obscured. Will alert user if
    site is not secure.
    """

    __slots__ = ()
    type = "password"


Password = PasswordInput


class RadioButton(Input):
    """A radio button, allowing a single value to be selected out of multiple
    choices with the same name value.
    """

    __slots__ = ()
    type = "radio"


Radio = RadioButton


class RangeSlider(Input):
    """A control for entering a number whose exact value is not important.
    Displays as a range widget defaulting to the middle value. Used in
    conjunction min and max to define the range of acceptable values.
    """

    __slots__ = ()
    type = "range"


Range = RangeSlider


class ResetButton(Input):
    """A button that resets the contents of the form to default values. Not
    recommended.
    """

    __slots__ = ()
    type = "reset"


Reset = ResetButton


class SearchBox(Input):
    """A single-line text field for entering search strings. Line-breaks are
    automatically removed from the input value. May include a delete icon in
    supporting browsers that can be used to clear the field. Displays a
    search icon instead of enter key on some devices with dynamic keypads.
    """

    __slots__ = ()
    type = "search"


Search = SearchBox


class SubmitButton(Input):
    """A button that submits the form."""

    __slots__ = ()
    type = "submit"


Submit = SubmitButton


class TelephoneInput(Input):
    """A control for entering a telephone number. Displays a telephone keypad
    in some devices with dynamic keypads.
    """

    __slots__ = ()
    type = "tel"


Tel = TelephoneInput


class TextBox(Input):
    """The default value. A single-line text field. Line-breaks are
    automatically removed from the input value.
    """

    __slots__ = ()
    type = "text"


Text = TextBox


class TimeInput(Input):
    """A control for entering a time value with no time zone."""

    __slots__ = ()
    type = "time"


Time = TimeInput


class URLBox(Input):
    """A field for entering a URL. Looks like a text input, but has validation
    parameters and relevant keyboard in supporting browsers and devices with
    dynamic keyboards.
    """

    __slots__ = ()
    type = "url"


URL = URLBox


class WeekInput(Input):
    """A control for entering a date consisting of a week-year number and a
    week number with no time zone.
    """

    __slots__ = ()
    type = "week"


Week = WeekInput


class Label(Container):
    """The HTML `<label>` element represents a caption for an item in a user
    interface.

    Use `for_` for the `for` parameter (to avoid keyword conflict)
    """

    __slots__ = ()
    tag = "label"


class Legend(Container):
    """The HTML `<legend>` element represents a caption for the content of
    its parent `<fieldset>`.
    """

    __slots__ = ()
    tag = "legend"


class Meter(Container):
    """The HTML <meter> element represents either a scalar value within a
    known range or a fractional value.

    ```html
    <label for="fuel">Fuel level:</label>

    <meter id="fuel"
        min="0" max="100"
        low="33" high="66" optimum="80"
        value="50">
        at 50/100
    </meter>
    ```
    """

    __slots__ = ()
    tag = "meter"


class OptionGroup(Container):
    """The HTML `<optgroup>` element creates a grouping of options within a
    `<select>` element.

    ```html
    <label for="dino-select">Choose a dinosaur:</label>
    <select id="dino-select">
        <optgroup label="Theropods">
            <option>Tyrannosaurus</option>
            <option>Velociraptor</option>
            <option>Deinonychus</option>
        </optgroup>
        <optgroup label="Sauropods">
            <option>Diplodocus</option>
            <option>Saltasaurus</option>
            <option>Apatosaurus</option>
        </optgroup>
    </select>
    ```
    """

    __slots__ = ()
    tag = "optgroup"


OptGroup = OptionGroup


class Option(Container):
    """The HTML `<option>` element is used to define an item contained in a
    `<select>`, an `<optgroup>`, or a `<datalist>` element. As such,
    `<option>` can represent menu items in popups and other lists of items
    in an HTML document.
    """

    __slots__ = ()
    tag = "option"


class Output(Container):
    """The HTML Output element (`<output>`) is a container element into which
    a site or app can inject the results of a calculation or the outcome of a
    user action.

    ## Attributes

    - `for`
        - A space-separated list of other elements’ `id`s, indicating that
            those elements contributed input values to (or otherwise
            affected) the calculation.
    - `form`
        - The `<form>` element to associate the output with (its form owner).
            The value of this attribute must be the `id` of a `<form>` in the
            same document. (If this attribute is not set, the `<output>` is
            associated with its ancestor `<form>` element, if any.)
        - This attribute lets you associate `<output>` elements to `<form>`s
            anywhere in the document, not just inside a `<form>`. It can also
            override an ancestor `<form>` element.
    - name
        - The element's name. Used in the `form.elements` API.
        - The `<output>` value, name, and contents are NOT submitted during
            form submission.

    ```html
    <form oninput="result.value=parseInt(a.value)+parseInt(b.value)">
        <input type="range" id="b" name="b" value="50" /> +
        <input type="number" id="a" name="a" value="10" /> =
        <output name="result" for="a b">60</output>
    </form>
    ```
    """

    __slots__ = ()
    tag = "output"


class Progress(Container):
    """The HTML `<progress>` element displays an indicator showing the
    completion progress of a task, typically displayed as a progress bar.

    ```html
    <label for="file">File progress:</label>

    <progress id="file" max="100" value="70">70%</progress>
    ```
    """

    __slots__ = ()
    tag = "progress"


class Select(Container):
    """The HTML `<select>` element represents a control that provides a menu
    of options

    ```html
    <label for="pet-select">Choose a pet:</label>

    <select name="pets" id="pet-select">
        <option value="">--Please choose an option--</option>
        <option value="dog">Dog</option>
        <option value="cat">Cat</option>
        <option value="hamster">Hamster</option>
        <option value="parrot">Parrot</option>
        <option value="spider">Spider</option>
        <option value="goldfish">Goldfish</option>
    </select>
    ```
    """

    __slots__ = ()
    tag = "select"


class TextArea(Container):
    """The HTML `<textarea>` element represents a multi-line plain-text
    editing control, useful when you want to allow users to enter a sizeable
    amount of free-form text, for example a comment on a review or feedback
    form.

    ```html
    <label for="story">Tell us your story:</label>

    <textarea id="story" name="story"
        rows="5" cols="33">
    It was a dark and stormy night...
    </textarea>
    ```
    """

    __slots__ = ()
    tag = "textarea"



from .base_classes import Container


class DisclosureSummary(Container):
    """The HTML Disclosure Summary element (`<summary>`) element specifies a
    summary, caption, or legend for a `<details>` element's disclosure box.
    """

    __slots__ = ()
    tag = "summary"


Summary = DisclosureSummary


class Details(Container):
    """The HTML Details Element (`<details>`) creates a disclosure widget in
    which information is visible only when the widget is toggled into an
    "open" state.
    """

    __slots__ = ()
    tag = "details"


class Dialogue(Container):
    """The HTML `<dialog>` element represents a dialogue box or other
    interactive component, such as a dismissable alert, inspector, or
    subwindow.
    """

    __slots__ = ()
    tag = "dialog"


Dialog = Dialogue


class Menu(Container):
    """The HTML `<menu>` element represents a group of commands that a user
    can perform or activate. This includes both list menus, which might appear
    across the top of a screen, as well as context menus, such as those that
    might appear underneath a button after it has been clicked.

    Context menus consist of a `<menu>` element which contains `<menuitem>`
    elements for each selectable option in the menu, `<menu>` elements for
    submenus within the menu, and `<hr>` elements for separator lines to break
    up the menu's content into sections. Context menus are then attached to
    the element they're activated from using either the associated element's
    `contextmenu` attribute or, for button-activated menus attached to
    `<button>` elements, the `menu` attribute.

    Toolbar menus consist of a `<menu>` element whose content is described in
    one of two ways: either as an unordered list of items represented by
    `<li>` elements (each representing a command or option the user can
    utilise), or (if there are no `<li>` elements), flow content describing
    the available commands and options.
    """

    __slots__ = ()
    tag = "menu"



import json
import typing
from abc import ABCMeta, abstractmethod
from textwrap import indent

Attributes = typing.Dict[str, typing.Any]
Content = typing.List[typing.Union['Element', str]]

def transform(attributes: Attributes) -> typing.Dict[str, str]:
    """Transform `attributes` for serialisation.

    `attributes` is not mutated; a new dictionary is returned.

    `classes`, if present, must be a `set` of classes to apply to the
    element. Empty strings will be removed from this set, so they are
    useable to toggle a class with a boolean.

    Any attribute name with an underscore in it has its underscores
    transformed into hyphens (for ARIA, JavaScript `data-` attributes,
    and similar) - use two underscores to represent a literal underscore.

    Additionally, single trailing underscores are removed.

    Attribute values are serialised as JSON, and non-strings are then
    serialised as JSON again (to be XHTML-compliant and avoid breaking
    arrays or objects as attribute values).
    """
    new_attributes = {}
    classes = attributes.pop("classes", set())
    if classes:
        attributes["class"] = " ".join(classes - {""})
    for key, value in attributes.items():
        if not isinstance(value, str):
            # convert value to a string containing JSON
            value = json.dumps(value)
        # convert value to a JSON-encoded string (double quotes, for HTML
        # attribute values)
        value = json.dumps(value.replace('"', "&quot;"))
        # remove single trailing underscore
        if key[-1] == "_" and key[-2] != "_":
            key = key[:-1]
        # replace double underscore with single underscore, and other
        # underscores with hyphens
        key = key.replace("__", " ").replace("_", "-").replace(" ", "_")
        new_attributes[key] = value
    return new_attributes


class Element(metaclass=ABCMeta):
    """Base class for all elements

    You should never need to inherit from this directly - use `Component` in
    most cases, or for custom elements use `Container` and `Void`

    When any element is created, its `apply_attributes` method is guaranteed
    to be called first, followed by its `set_content` method.

    Note: If the element is then called (HTML-style content initialisation)
    then its `set_content` method will be called again
    """

    __slots__ = ()

    def __init__(
        self, *content: typing.Union["Element", str], **attributes: typing.Any
    ) -> None:
        self.apply_attributes(attributes)
        self.set_content(list(content))

    def __call__(self, *content: typing.Union["Element", str]) -> "Element":
        self.set_content(list(content))
        return self

    @abstractmethod
    def serialise(self, minify: bool = True) -> str:
        """Return an HTML representation of the model"""

    @abstractmethod
    def set_content(self, content: Content) -> None:
        """Set content - called by __init__ and __call__"""

    @abstractmethod
    def apply_attributes(self, attributes: Attributes) -> None:
        """Apply attributes - called by __init__

        Guaranteed to be called before set_content"""

    def __str__(self) -> str:
        return self.serialise(minify=False)

class TextElement(Element):
    __slots__ = ("content",)
    content: str

    def serialise(self, minify: bool = True) -> str:
        """Return an HTML representation of the model"""
        return self.content

    def set_content(self, content: Content) -> None:
        self.content = content[0]
        if not isinstance(self.content, str):
            raise TypeError(
                "Text content must be a single string (got {!r})".format(
                    type(self.content).__qualname__
                )
            )
        if len(content) != 1:
            raise TypeError(
                "Text content must be a single string (got {} items)".format(
                    len(content)
                )
            )

    def apply_attributes(self, attributes: Attributes) -> None:
        if attributes:
            raise TypeError("Text content cannot have attributes")

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self.content!r})"


class Container(Element):
    """Base class for all container/non-void elements

    Most inheriters only need to define `tag`
    """

    __slots__ = ("tag", "content", "attributes")
    tag: str
    content: typing.List["Element"]
    attributes: Attributes

    def serialise(self, minify: bool = True) -> str:
        """Return an HTML representation of the model"""
        return (
            "<"
            + self.tag
            + (
                (" " + " ".join(f"{k}={v}" for k, v in self.attributes.items()))
                if self.attributes
                else ""
            )
            + ">"
            + ("" if minify else "\n")
            + indent(
                ("" if minify else "\n").join(
                    child.serialise(minify=minify) for child in self.content
                ),
                "    ",
                lambda i: not minify,
            )
            + ("" if minify else "\n")
            + f"</{self.tag}>"
        )

    def set_content(self, content: Content) -> None:
        self.content = [
            child if isinstance(child, Element) else TextElement(child)
            for child in content
        ]

    def apply_attributes(self, attributes: Attributes) -> None:
        self.attributes = transform(attributes)

    def __repr__(self) -> str:
        return (
            self.__class__.__name__
            + "("
            + ", ".join(repr(child) for child in self.content)
            + (", " if self.content and self.attributes else "")
            + ", ".join(f"{k}={v!r}" for k, v in self.attributes.items())
            + ")"
        )


class Void(Element):
    """Base class for all non-container/void elements

    Most inheriters only need to define `tag`
    """

    __slots__ = ("tag", "attributes")
    tag: str
    attributes: Attributes

    def serialise(self, minify: bool = True) -> str:
        """Return an HTML representation of the model"""
        return (
            "<"
            + self.tag
            + (
                (" " + " ".join(f"{k}={v}" for k, v in self.attributes.items()))
                if self.attributes
                else ""
            )
            + " />"
        )

    def set_content(self, content: Content) -> None:
        if content:
            raise TypeError("Void elements cannot have content")

    def apply_attributes(self, attributes: Attributes) -> None:
        self.attributes = transform(attributes)

    def __repr__(self) -> str:
        return (
            self.__class__.__name__
            + "("
            + ", ".join(f"{k}={v}" for k, v in self.attributes.items())
            + ")"
        )


class ElementGroup(Element):
    """A group of elements. Useful for creating Components"""

    __slots__ = ("content",)
    content: typing.List[Element]

    def set_content(self, content: Content) -> None:
        self.content = [
            child if isinstance(child, Element) else TextElement(child)
            for child in content
        ]

    def apply_attributes(self, attributes: Attributes) -> None:
        if attributes:
            raise TypeError("Element groups cannot have attributes")

    def serialise(self, minify: bool) -> str:
        return ("" if minify else "\n").join(
            child.serialise(minify=minify) for child in self.content
        )


class Component(Element):
    """Base class for custom components

    `apply_attributes` is always called before `set_content` so it can be
    used to control the behaviour of the component
    """

    __slots__ = ("content",)
    content: Element

    def serialise(self, minify: bool = True) -> str:
        """Return an HTML representation of the model"""
        return self.content.serialise(minify=minify)



import functools

from .base_classes import Container, Void


class BaseURL(Void):
    """The HTML `<base>` element specifies the base URL to use for *all*
    relative URLs in a document. There can be only one `<base>` element in a
    document.
    """

    __slots__ = ()
    tag = "base"


Base = BaseURL


class ExternalResourceLink(Void):
    """The HTML External Resource Link element (`<link>`) specifies
    relationships between the current document and an external resource.
    This element is most commonly used to link to stylesheets, but is
    also used to establish site icons (both "favicon" style icons and
    icons for the home screen and apps on mobile devices) among other
    things.
    """

    __slots__ = ()
    tag = "link"


Link = ExternalResourceLink

ExternalStyleSheet = functools.partial(ExternalResourceLink, rel="stylesheet")


class Meta(Void):
    """The HTML `<meta>` element represents metadata that cannot be
    represented by other HTML meta-related elements, like `<base>`,
    `<link>`, `<script>`, `<style>` or `<title>`.
    """

    __slots__ = ()
    tag = "meta"


class Style(Container):
    """The HTML `<style>` element contains style information for a
    document, or part of a document.
    """

    __slots__ = ()
    tag = "style"


class Title(Container):
    """The HTML Title element (`<title>`) defines the document's title
    that is shown in a browser's title bar or a page's tab.
    """

    __slots__ = ()
    tag = "title"



import typing

from .base_classes import Container, Void


class Area(Void):
    """The HTML `<area>` element defines a hot-spot region on an image, and
    optionally associates it with a hypertext link. This element is used only
    within a `<map>` element.

    ```html
    <map name="infographic">
        <area shape="rect" coords="184,6,253,27"
            href="https://mozilla.org"
            target="_blank" alt="Mozilla" />
        <area shape="circle" coords="130,136,60"
            href="https://developer.mozilla.org/"
            target="_blank" alt="MDN" />
        <area shape="poly" coords="130,6,253,96,223,106,130,39"
            href="https://developer.mozilla.org/docs/Web/Guide/Graphics"
            target="_blank" alt="Graphics" />
        <area shape="poly" coords="253,96,207,241,189,217,223,103"
            href="https://developer.mozilla.org/docs/Web/HTML"
            target="_blank" alt="HTML" />
        <area shape="poly" coords="207,241,54,241,72,217,189,217"
            href="https://developer.mozilla.org/docs/Web/JavaScript"
            target="_blank" alt="JavaScript" />
        <area shape="poly" coords="54,241,6,97,36,107,72,217"
            href="https://developer.mozilla.org/docs/Web/API"
            target="_blank" alt="Web APIs" />
        <area shape="poly" coords="6,97,130,6,130,39,36,107"
            href="https://developer.mozilla.org/docs/Web/CSS"
            target="_blank" alt="CSS" />
    </map>
    <img usemap="#infographic" src="/media/examples/mdn-info.png"
        alt="MDN infographic" />
    ```
    """

    __slots__ = ()
    tag = "area"


class AudioPlayer(Container):
    """The HTML `<audio>` element is used to embed sound content in documents.
    It may contain one or more audio sources, represented using the src
    attribute or the `<source>` element: the browser will choose the most
    suitable one. It can also be the destination for streamed media, using a
    `MediaStream`.

    ```html
    <audio
        controls
        src="/media/cc0-audio/t-rex-roar.mp3">
            Your browser does not support the
            <code>audio</code> element.
    </audio>
    ```
    """

    __slots__ = ()
    tag = "audio"


Audio = AudioPlayer


class VideoPlayer(Container):
    """The HTML Video element (`<video>`) embeds a media player which supports
    video playback into the document. You can use `<video>` for audio content
    as well, but the `<audio>` element may provide a more appropriate user
    experience.
    """

    __slots__ = ()
    tag = "video"


Video = VideoPlayer


class Image(Void):
    """The HTML `<img>` element embeds an image into the document."""

    __slots__ = ()
    tag = "img"

    def apply_attributes(self, attributes: typing.Dict[str, typing.Any]) -> None:
        return super().apply_attributes(attributes)


Img = Image


class ImageMap(Container):
    """The HTML `<map>` element is used with `<area>` elements to define an
    image map (a clickable link area).
    """

    __slots__ = ()
    tag = "map"


Map = ImageMap


class Path(Void):
    __slots__ = ()
    tag = "path"


class ScalableVectorGraphic(Container):
    __slots__ = ()
    tag = "svg"


SVG = ScalableVectorGraphic


class Polygon(Container):
    __slots__ = ()
    tag = "polygon"


class Track(Void):
    """The HTML `<track>` element is used as a child of the media elements,
    `<audio>` and `<video>`. It lets you specify timed text tracks (or
    time-based data), for example to automatically handle subtitles. The
    tracks are formatted in WebVTT format (.vtt files) — Web Video Text Tracks

    ```html
    <video controls
        src="/media/cc0-videos/friday.mp4">
        <track default
            kind="captions"
            srclang="en"
            src="/media/examples/friday.vtt" />
        Sorry, your browser doesn't support embedded videos.
    </video>
    ```

    ## Attributes

    - `default`
        - This attribute indicates that the track should be enabled unless the
            user's preferences indicate that another track is more
            appropriate. This may only be used on one `track` element per
            media element.
    - `kind`
        - How the text track is meant to be used. If omitted the default kind
            is `subtitles`. If the attribute contains an invalid value, it
            will use `metadata` (Versions of Chrome earlier than 52 treated an
            invalid value as `subtitles`). The following keywords are allowed:
        - `subtitles`
            - Subtitles provide translation of content that cannot be
                understood by the viewer. For example dialogue or text that
                is not English in an English language film.
            - Subtitles may contain additional content, usually extra
                background information. For example the text at the beginning
                of the Star Wars films, or the date, time, and location of a
                scene.
        - `captions`
            - Closed captions provide a transcription and possibly a
                translation of audio.
            - It may include important non-verbal information such as music
                cues or sound effects. It may indicate the cue's source (e.g.
                music, text, character).
            - Suitable for users who are deaf or when the sound is muted.
        - `descriptions`
            - Textual description of the video content.
            - Suitable for users who are blind or where the video cannot be
                seen.
        - `chapters`
            - Chapter titles are intended to be used when the user is
                navigating the media resource.
        - `metadata`
            - Tracks used by scripts. Not visible to the user.
    - `label`
        - A user-readable title of the text track which is used by the browser
            when listing available text tracks.
    - `src`
        - Address of the track (`.vtt` file). Must be a valid URL. This
            attribute must be specified and its URL value must have the same
            origin as the document — unless the `<audio>` or `<video>` parent
            element of the `track` element has a `crossorigin` attribute.
    - `srclang`
        - Language of the track text data. It must be a valid BCP 47 language
            tag. If the `kind` attribute is set to `subtitles`, then `srclang`
            must be defined.
    """

    __slots__ = ()
    tag = "track"



import typing

from .base_classes import Container, Element, TextElement, Void


class Anchor(Container):
    """The HTML `<a>` element (or anchor element), with its `href`
    attribute, creates a hyperlink to web pages, files, email addresses,
    locations in the same page, or anything else a URL can address.
    """

    __slots__ = ()
    tag = "a"


A = Anchor


class Abbreviation(Container):
    """The HTML Abbreviation element (`<abbr>`) represents an abbreviation
    or acronym; the optional `title` attribute can provide an expansion or
    description for the abbreviation.
    """

    __slots__ = ()
    tag = "abbr"


Abbr = Abbreviation


class BringAttentionTo(Container):
    """The HTML Bring Attention To element (`<b>`) is used to draw the
    reader's attention to the element's contents, which are not otherwise
    granted special importance.
    """

    __slots__ = ()
    tag = "b"


B = BringAttentionTo


class BidirectionalIsolateElement(Container):
    """The HTML Bidirectional Isolate element (`<bdi>`) tells the browser's
    bidirectional algorithm to treat the text it contains in isolation
    from its surrounding text.
    """

    __slots__ = ()
    tag = "bdi"


BDI = BidirectionalIsolateElement


class BidirectionalTextOverride(Container):
    """The HTML Bidirectional Text Override element (`<bdo>`) overrides the
    current directionality of text, so that the text within is rendered in a
    different direction.
    """

    __slots__ = ()
    tag = "bdo"


BDO = BidirectionalTextOverride


class LineBreak(Void):
    """The HTML `<br>` element produces a line break in text (carriage-return).
    It is useful for writing a poem or an address, where the division of lines
    is significant.
    """

    __slots__ = ()
    tag = "br"


Br = LineBreak


class Citation(Container):
    """The HTML Citation element (`<cite>`) is used to describe a reference
    to a cited creative work, and must include the title of that work.
    """

    __slots__ = ()
    tag = "cite"


Cite = Citation


class Code(Container):
    """The HTML `<code>` element displays its contents styled in a fashion
    intended to indicate that the text is a short fragment of computer code.
    """

    __slots__ = ()
    tag = "code"


class Data(Container):
    """The HTML `<data>` element links a given piece of content with a
    machine-readable translation. If the content is time- or date-related,
    the `<time>` element must be used.

    Example:
    ```html
    <p>New Products</p>
    <ul>
        <li><data value="398">Mini Ketchup</data></li>
        <li><data value="399">Jumbo Ketchup</data></li>
        <li><data value="400">Mega Jumbo Ketchup</data></li>
    </ul>
    ```
    """

    __slots__ = ()
    tag = "data"


class Definition(Container):
    """The HTML Definition element (`<dfn>`) is used to indicate the term
    being defined within the context of a definition phrase or sentence.
    """

    __slots__ = ()
    tag = "dfn"


Dfn = Definition


class Emphasis(Container):
    """The HTML `<em>` element marks text that has stress emphasis. The
    `<em>` element can be nested, with each level of nesting indicating a
    greater degree of emphasis.
    """

    __slots__ = ()
    tag = "em"


Em = Emphasis


class IdiomaticText(Container):
    """The HTML Idiomatic Text element (`<i>`) represents a range of text
    that is set off from the normal text for some reason, such as idiomatic
    text, technical terms, taxonomical designations, among others.
    """

    __slots__ = ()
    tag = "i"


I = IdiomaticText


class KeyboardInput(Container):
    """The HTML Keyboard Input element (`<kbd>`) represents a span of inline
    text denoting textual user input from a keyboard, voice input, or any
    other text entry device.
    """

    __slots__ = ()
    tag = "kbd"


Kbd = KeyboardInput


class MarkText(Container):
    """The HTML Mark Text element (`<mark>`) represents text which is marked
    or highlighted for reference or notation purposes, due to the marked
    passage's relevance or importance in the enclosing context.
    """

    __slots__ = ()
    tag = "mark"


Mark = MarkText


class InlineQuotation(Container):
    """The HTML `<q>` element indicates that the enclosed text is a short
    inline quotation. Most modern browsers implement this by surrounding
    the text in quotation marks.
    """

    __slots__ = ()
    tag = "q"


Q = InlineQuotation


class RubyText(Container):
    """The HTML Ruby Text (`<rt>`) element specifies the ruby text component
    of a ruby annotation, which is used to provide pronunciation, translation,
    or transliteration information for East Asian typography. The `<rt>`
    element must always be contained within a `<ruby>` element.
    """

    __slots__ = ()
    tag = "rt"


RT = RubyText


class RubyFallback(Container):
    """The HTML Ruby Fallback Parenthesis (`<rp>`) element is used to provide
    fall-back parentheses for browsers that do not support display of ruby
    annotations using the `<ruby>` element.
    """

    __slots__ = ()
    tag = "rp"


RP = RubyFallback


class Ruby(Container):
    """The HTML `<ruby>` element represents small annotations that are
    rendered above, below, or next to base text, usually used for showing
    the pronunciation of East Asian characters. It can also be used for
    annotating other kinds of text, but this usage is less common.
    """

    __slots__ = ()
    tag = "ruby"


class Strikethrough(Container):
    """The HTML `<s>` element renders text with a strikethrough, or a line
    through it. Use the `<s>` element to represent things that are no longer
    relevant or no longer accurate. However, `<s>` is not appropriate when
    indicating document edits; for that, use the `<del>` and `<ins>` elements,
    as appropriate.
    """

    __slots__ = ()
    tag = "s"


S = Strikethrough


class SampleOutput(Container):
    """The HTML Sample Element (`<samp>`) is used to enclose inline text which
    represents sample (or quoted) output from a computer program.
    """

    __slots__ = ()
    tag = "samp"


Samp = SampleOutput


class Small(Container):
    """The HTML `<small>` element represents side-comments and small print,
    like copyright and legal text, independent of its styled presentation. By
    default, it renders text within it one font-size smaller, such as from
    small to x-small.
    """

    __slots__ = ()
    tag = "small"


class Span(Container):
    """The HTML `<span>` element is a generic inline container for phrasing
    content, which does not inherently represent anything. It can be used to
    group elements for styling purposes (using the `class` or `id` attributes),
    or because they share attribute values, such as `lang`.
    """

    __slots__ = ()
    tag = "span"


class StrongImportance(Container):
    """The HTML Strong Importance Element (`<strong>`) indicates that its
    contents have strong importance, seriousness, or urgency. Browsers
    typically render the contents in bold type.
    """

    __slots__ = ()
    tag = "strong"


Strong = StrongImportance


class Subscript(Container):
    """The HTML Subscript element (`<sub>`) specifies inline text which
    should be displayed as subscript for solely typographical reasons.
    """

    __slots__ = ()
    tag = "sub"


Sub = Subscript


class Superscript(Container):
    """The HTML Superscript element (`<sup>`) specifies inline text which
    is to be displayed as superscript for solely typographical reasons.
    """

    __slots__ = ()
    tag = "sup"


Sup = Superscript


class Time(Container):
    """The HTML <time> element represents a specific period in time.

    ```html
    <time datetime="2018-07-07">7th July</time>
    <time datetime="20:00">20:00</time>
    <time datetime="PT2H30M">2h30m</time>
    ```

    ## Valid datetime Values

    - a valid year string
        - `2011`
    - a valid month string
        - `2011-11`
    - a valid date string
        - `2011-11-18`
    - a valid yearless date string
        - `11-18`
    - a valid week string
        - `2011-W47`
    - a valid time string
        - `14:54`
    14:54:39
    14:54:39.929
    - a valid local date and time string
        - `2011-11-18T14:54:39.929`
    2011-11-18 14:54:39.929
    - a valid global date and time string
        - `2011-11-18T14:54:39.929Z`
        - `2011-11-18T14:54:39.929-0400`
        - `2011-11-18T14:54:39.929-04:00`
        - `2011-11-18 14:54:39.929Z`
        - `2011-11-18 14:54:39.929-0400`
        - `2011-11-18 14:54:39.929-04:00`
    - a valid duration string
        - `PT4H18M3S`
    """

    __slots__ = ()
    tag = "time"


class UnarticulatedAnnotation(Container):
    """The HTML Unarticulated Annotation element (`<u>`) represents a span of
    inline text which should be rendered in a way that indicates that it has a
    non-textual annotation.
    """

    __slots__ = ()
    tag = "u"


U = UnarticulatedAnnotation


class Variable(Container):
    """The HTML Variable element (`<var>`) represents the name of a variable
    in a mathematical expression or a programming context.
    """

    __slots__ = ()
    tag = "var"


Var = Variable


class WordBreak(Void):
    """The HTML `<wbr>` element represents a word break opportunity — a
    position within text where the browser may optionally break a line, though
    its line-breaking rules would not otherwise create a break at that
    location.
    """

    __slots__ = ()
    tag = "wbr"


WBr = WordBreak


class DeletedText(Container):
    """The HTML `<del>` element represents a range of text that has been
    deleted from a document.
    """

    __slots__ = ()
    tag = "del"


Del = DeletedText


class InsertedText(Container):
    """The HTML `<ins>` element represents a range of text that has been
    added to a document.
    """

    __slots__ = ()
    tag = "ins"


Ins = InsertedText



import typing

from .. import plain_elements as dom

_INITIALISER_CSS_URLS = {"https://starwort.github.io/domgen/cdn/styles/common.css"}
_FINALISER_JS_URLS = set()


def component_register(css_url: str, js_url: str) -> None:
    """MDC components will call this function during their initialisation to
    register with the module variables above. Using this function manually is
    not recommended, but will probably work to register both CSS and finalising
    JS to be included with the MDC loader components"""
    _INITIALISER_CSS_URLS.add(css_url)
    _FINALISER_JS_URLS.add(js_url)


def clear_component_cache():
    """Use this to clear the cache of used components.

    Useful for if the same Python process is used to generate several pages
    using Material elements.
    """
    _INITIALISER_CSS_URLS.clear()
    _INITIALISER_CSS_URLS.add("https://starwort.github.io/domgen/cdn/styles/common.css")
    _FINALISER_JS_URLS.clear()


class MDCInitialiser(dom.Component):
    """Place this in the head of your document to include the required CSS.

    Ensure that your page's `Body` element has a `theme` attribute set to
    either `'light'` or `'dark'`.

    Expects no content, and as such any content passed will be ignored.
    """

    __slots__ = ()

    def apply_attributes(self, attributes: typing.Dict[str, typing.Any]) -> None:
        extra_kws = set(attributes.keys())
        assert extra_kws == set(), "Unexpected keyword arguments passed: " + ", ".join(
            extra_kws
        )

    def set_content(
        self, content: typing.List[typing.Union[dom.Element, str]] = None
    ) -> None:
        self.content = dom.ElementGroup(
            *[dom.ExternalStyleSheet(href=css_url) for css_url in _INITIALISER_CSS_URLS]
        )

    def serialise(self, minify: bool) -> str:
        self.set_content()
        return super().serialise(minify=minify)


class MDCFinaliser(dom.Component):
    """Place this at the bottom of your document to include the required JS.

    Expects no content, and as such any content passed will be ignored.
    """

    __slots__ = ()

    def apply_attributes(self, attributes: typing.Dict[str, typing.Any]) -> None:
        if attributes:
            raise TypeError("MDCFinaliser components take no attributes")

    def set_content(
        self, content: typing.List[typing.Union[dom.Element, str]] = None
    ) -> None:
        self.content = dom.ElementGroup(
            *[dom.Script(src=js_url) for js_url in _FINALISER_JS_URLS]
        )

    def serialise(self, minify: bool) -> str:
        self.set_content()
        return super().serialise(minify=minify)



import typing

from .. import plain_elements as dom
from . import component_register


class MenuButton(dom.Component):
    __slots__ = ("attributes", "classes")

    def apply_attributes(self, attributes: dom.Attributes) -> None:
        self.classes = attributes.pop("classes", set())
        self.classes |= {
            "material-icons",
            "mdc-top-app-bar__navigation-icon",
            "mdc-icon-button",
        }
        self.attributes = attributes
        if "aria_label" not in self.attributes:
            self.attributes["aria_label"] = "Open navigation menu"

    def set_content(self, content: dom.Content) -> None:
        self.content = dom.Button(classes=self.classes, **self.attributes)(
            *(content or ["menu"])
        )


class _AppBarBase(dom.Component):
    __slots__ = ("menu_button", "title", "toolbar_buttons", "navigation_drawer")
    menu_button: MenuButton
    title: dom.Content
    toolbar_buttons: dom.Content
    navigation_drawer: dom.Element

    def apply_attributes(self, attributes: dom.Attributes) -> None:
        self.menu_button = attributes.get("menu_button", None)
        self.title = attributes.get("title", [])
        self.toolbar_buttons = attributes.get("toolbar_buttons", [])
        self.navigation_drawer = attributes.get("navigation_drawer", None)


class _Standard(_AppBarBase):
    __slots__ = ()

    def set_content(self, content: dom.Content) -> None:
        self.content = dom.ElementGroup(
            dom.Header(classes={"mdc-top-app-bar"})(
                dom.ContentDivision(classes={"mdc-top-app-bar__row"})(
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-start",
                        }
                    )(
                        self.menu_button or "",
                        dom.Span(classes={"mdc-top-app-bar__title"})(*self.title),
                    ),
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-end",
                        },
                        role="toolbar",
                    )(**self.toolbar_buttons),
                )
            ),
            self.navigation_drawer if self.navigation_drawer else "",
            dom.MainContent(
                classes={
                    "mdc-top-app-bar--fixed-adjust",
                    "mdc-drawer-app-content" if self.navigation_drawer else "",
                }
            )(*content),
        )


class _Short(_AppBarBase):
    __slots__ = ("collapsed",)
    collapsed: bool

    def apply_attributes(self, attributes: dom.Attributes) -> None:
        self.collapsed = attributes.get("collapsed", False)
        return super().apply_attributes(attributes)

    def set_content(self, content: dom.Content) -> None:
        self.content = dom.ElementGroup(
            dom.Header(
                classes={
                    "mdc-top-app-bar",
                    "mdc-top-app-bar--short",
                    "mdc-top-app-bar--short-collapsed" if self.collapsed else "",
                }
            )(
                dom.ContentDivision(classes={"mdc-top-app-bar__row"})(
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-start",
                        }
                    )(
                        self.menu_button or "",
                        dom.Span(classes={"mdc-top-app-bar__title"})(*self.title),
                    ),
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-end",
                        },
                        role="toolbar",
                    )(**self.toolbar_buttons),
                )
            ),
            self.navigation_drawer if self.navigation_drawer else "",
            dom.MainContent(
                classes={
                    "mdc-top-app-bar--short-fixed-adjust",
                    "mdc-drawer-app-content" if self.navigation_drawer else "",
                }
            )(*content),
        )


class _Fixed(_AppBarBase):
    __slots__ = ()

    def set_content(self, content: dom.Content) -> None:
        self.content = dom.ElementGroup(
            dom.Header(classes={"mdc-top-app-bar", "mdc-top-app-bar--fixed"})(
                dom.ContentDivision(classes={"mdc-top-app-bar__row"})(
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-start",
                        }
                    )(
                        self.menu_button or "",
                        dom.Span(classes={"mdc-top-app-bar__title"})(*self.title),
                    ),
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-end",
                        },
                        role="toolbar",
                    )(**self.toolbar_buttons),
                )
            ),
            self.navigation_drawer if self.navigation_drawer else "",
            dom.MainContent(
                classes={
                    "mdc-top-app-bar--fixed-adjust",
                    "mdc-drawer-app-content" if self.navigation_drawer else "",
                }
            )(*content),
        )


class _Prominent(_AppBarBase):
    __slots__ = ()

    def set_content(self, content: dom.Content) -> None:
        self.content = dom.ElementGroup(
            dom.Header(classes={"mdc-top-app-bar", "mdc-top-app-bar--prominent"})(
                dom.ContentDivision(classes={"mdc-top-app-bar__row"})(
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-start",
                        }
                    )(
                        self.menu_button or "",
                        dom.Span(classes={"mdc-top-app-bar__title"})(*self.title),
                    ),
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-end",
                        },
                        role="toolbar",
                    )(**self.toolbar_buttons),
                )
            ),
            self.navigation_drawer if self.navigation_drawer else "",
            dom.MainContent(
                classes={
                    "mdc-top-app-bar--prominent-fixed-adjust",
                    "mdc-drawer-app-content" if self.navigation_drawer else "",
                }
            )(*content),
        )


class _Dense(_AppBarBase):
    __slots__ = ()

    def set_content(self, content: dom.Content) -> None:
        self.content = dom.ElementGroup(
            dom.Header(classes={"mdc-top-app-bar", "mdc-top-app-bar--dense"})(
                dom.ContentDivision(classes={"mdc-top-app-bar__row"})(
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-start",
                        }
                    )(
                        self.menu_button or "",
                        dom.Span(classes={"mdc-top-app-bar__title"})(*self.title),
                    ),
                    dom.Section(
                        classes={
                            "mdc-top-app-bar__section",
                            "mdc-top-app-bar__section--align-end",
                        },
                        role="toolbar",
                    )(**self.toolbar_buttons),
                )
            ),
            self.navigation_drawer if self.navigation_drawer else "",
            dom.MainContent(
                classes={
                    "mdc-top-app-bar--dense-fixed-adjust",
                    "mdc-drawer-app-content" if self.navigation_drawer else "",
                }
            )(*content),
        )


class TopAppBar(dom.Component):
    """Top App Bar.

    https://material.io/components/app-bars-top/

    Comes in 5 variants:
      - Standard
      - Short
      - Fixed
      - Prominent
      - Dense

    Takes the following parameters:
      - `variant`: One of `'standard'`, `'short'`, `'fixed'`, `'prominent'`,
        and `'dense'`. Selects the kind of top app bar to use.
      - `menu_button`: If present, should be an instance of MenuButton. Used
        to activate the menu. No functionality is provided by default.
      - `title`: Text (or elements) to display as the page title.
      - `toolbar_buttons`: A collection of elements to display as buttons in
        the app bar.
      - `navigation_drawer`: If present, should be an instance of
        NavigationDrawer.
      - `collapsed`: True to always collapse the app bar (Short app bar only).
    """

    __slots__ = ()
    content: _AppBarBase

    def apply_attributes(self, attributes: dom.Attributes) -> None:
        component_register(
            "https://starwort.github.io/domgen/cdn/styles/top_app_bar.css",
            "https://starwort.github.io/domgen/cdn/javascript/top_app_bar.js",
        )
        self.content = {
            "standard": _Standard,
            "short": _Short,
            "fixed": _Fixed,
            "prominent": _Prominent,
            "dense": _Dense,
        }[attributes["variant"]](**attributes)

    def set_content(self, content: dom.Content) -> None:
        self.content.set_content(content)



from .base_classes import Container


class Slot(Container):
    """The HTML `<slot>` element — part of the Web Components technology suite
    — is a placeholder inside a web component that you can fill with your own
    markup, which lets you create separate DOM trees and present them
    together.
    """

    __slots__ = ()
    tag = "slot"


class ContentTemplate(Container):
    """The HTML Content Template (`<template>`) element is a mechanism for
    holding HTML that is not to be rendered immediately when a page is loaded
    but may be instantiated subsequently during runtime using JavaScript.
    """

    __slots__ = ()
    tag = "template"


Template = ContentTemplate



from .components import Slot, Template
from .content import DD, DL, DT, HR, OL, UL, BlockQuote, Div, FigCaption, Figure, P, Pre
from .forms import (
    URL,
    Button,
    Checkbox,
    Color,
    DataList,
    Date,
    DatetimeLocal,
    Email,
    FieldSet,
    File,
    Form,
    Hidden,
    Image,
    Input,
    Label,
    Legend,
    Meter,
    Month,
    Number,
    OptGroup,
    Option,
    Output,
    Password,
    Progress,
    Radio,
    Range,
    Reset,
    Search,
    Select,
    Submit,
    Tel,
    Text,
    TextArea,
    TimeInput,
    Week,
)
from .interactive_elements import Details, Dialog, Menu, Summary
from .media import SVG, Area, Audio, Img, Map, Path, Polygon, Track, Video
from .metadata import Base, Link, Meta, Style, Title
from .scripts import Canvas, NoScript, Script
from .sectioning import (
    H1,
    H2,
    H3,
    H4,
    H5,
    H6,
    HTML,
    Address,
    Article,
    Aside,
    Body,
    Footer,
    Head,
    Header,
    HGroup,
    Main,
    Nav,
    Section,
)
from .semantics import (
    BDI,
    BDO,
    RP,
    RT,
    A,
    Abbr,
    B,
    Br,
    Cite,
    Code,
    Data,
    Del,
    Dfn,
    Em,
    I,
    Ins,
    Kbd,
    Mark,
    Q,
    Ruby,
    S,
    Samp,
    Small,
    Span,
    Strong,
    Sub,
    Sup,
    Time,
    U,
    Var,
    WBr,
)
from .tables import TD, TH, TR, Caption, Col, ColGroup, Table, TBody, TFoot, THead



import setuptools

import dom

with open("README.md", "r") as file:
    long_description = file.read()

setuptools.setup(
    name="domgen",
    version=dom.__version__,
    author="Starwort",
    description="A package for generating HTML and CSS",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Starwort/domgen",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[],
    extras_require={},
)



from . import (
    base_classes,
    components,
    forms,
    interactive_elements,
    material_design,
    media,
    metadata,
    scripts,
    sectioning,
    semantics,
    short,
    tables,
)
from .plain_elements import *

__version__ = "0.2.0a"



from .base_classes import (
    Attributes,
    Component,
    Container,
    Content,
    Element,
    ElementGroup,
    TextElement,
    Void,
    transform,
)
from .components import ContentTemplate, Slot
from .content import (
    BlockQuotation,
    ContentDivision,
    DescriptionDetails,
    DescriptionList,
    DescriptionTerm,
    Divider,
    Figure,
    FigureCaption,
    OrderedList,
    Paragraph,
    PreformattedText,
    UnorderedList,
)
from .forms import (
    Button,
    Checkbox,
    ColorInput,
    ColourInput,
    DataList,
    DateInput,
    DatetimeInput,
    EmailInput,
    FieldSet,
    FileInput,
    Form,
    HiddenInput,
    ImageButton,
    Input,
    Label,
    Legend,
    Meter,
    MonthInput,
    NumberInput,
    Option,
    OptionGroup,
    Output,
    PasswordInput,
    Progress,
    RadioButton,
    RangeSlider,
    ResetButton,
    SearchBox,
    Select,
    SubmitButton,
    TelephoneInput,
    TextArea,
    TextBox,
    TimeInput,
    URLBox,
    WeekInput,
)
from .interactive_elements import Details, Dialogue, DisclosureSummary, Menu
from .media import (
    Area,
    AudioPlayer,
    Image,
    ImageMap,
    Path,
    Polygon,
    ScalableVectorGraphic,
    Track,
    VideoPlayer,
)
from .metadata import (
    BaseURL,
    ExternalResourceLink,
    ExternalStyleSheet,
    Meta,
    Style,
    Title,
)
from .scripts import Canvas, NoScript, Script
from .sectioning import (
    HTML,
    ArticleContents,
    Aside,
    Body,
    ContactAddress,
    Footer,
    Head,
    Header,
    Heading1,
    Heading2,
    Heading3,
    Heading4,
    Heading5,
    Heading6,
    HeadingGroup,
    MainContent,
    Navigation,
    Section,
)
from .semantics import (
    Abbreviation,
    Anchor,
    BidirectionalIsolateElement,
    BidirectionalTextOverride,
    BringAttentionTo,
    Citation,
    Code,
    Data,
    Definition,
    DeletedText,
    Emphasis,
    IdiomaticText,
    InlineQuotation,
    InsertedText,
    KeyboardInput,
    LineBreak,
    MarkText,
    Ruby,
    RubyFallback,
    RubyText,
    SampleOutput,
    Small,
    Span,
    Strikethrough,
    StrongImportance,
    Subscript,
    Superscript,
    Time,
    UnarticulatedAnnotation,
    Variable,
    WordBreak,
)
from .tables import (
    ColumnDeclaration,
    ColumnGroup,
    Table,
    TableBody,
    TableCaption,
    TableDataCell,
    TableFoot,
    TableHead,
    TableHeaderCell,
    TableRow,
)



from .base_classes import Container, Void


class BlockQuotation(Container):
    """The HTML `<blockquote>` Element (or HTML Block Quotation Element)
    indicates that the enclosed text is an extended quotation. Usually,
    this is rendered visually by indentation (see Notes for how to change
    it). A URL for the source of the quotation may be given using the cite
    attribute, while a text representation of the source can be given using
    the `<cite>` element.
    """

    __slots__ = ()
    tag = "blockquote"


BlockQuote = BlockQuotation


class DescriptionTerm(Container):
    """The HTML `<dt>` element specifies a term in a description or definition
    list, and as such must be used inside a `<dl>` element. It is usually followed
    by a `<dd>` element; however, multiple `<dt>` elements in a row indicate several
    terms that are all defined by the immediate next `<dd>` element.
    """

    tag = "dt"


DT = DescriptionTerm


class DescriptionDetails(Container):
    """The HTML `<dd>` element provides the description, definition, or value for
    the preceding term (`<dt>`) in a description list (`<dl>`).
    """

    tag = "dd"


DD = DescriptionDetails


class DescriptionList(Container):
    """The HTML `<dl>` element represents a description list. The element
    encloses a list of groups of terms (specified using the `<dt>` element)
    and descriptions (provided by `<dd>` elements). Common uses for this
    element are to implement a glossary or to display metadata (a list of
    key-value pairs).
    """

    __slots__ = ()


DL = DescriptionList


class ContentDivision(Container):
    """The HTML Content Division element (`<div>`) is the generic container
    for flow content. It has no effect on the content or layout until styled
    using CSS.
    """

    __slots__ = ()
    tag = "div"


Div = ContentDivision


class Figure(Container):
    """The HTML `<figure>` (Figure With Optional Caption) element represents
    self-contained content, potentially with an optional caption, which is
    specified using the FigureCaption (`<figcaption>`) element.
    """

    __slots__ = ()
    tag = "figure"


class FigureCaption(Container):
    """The HTML `<figcaption>` or Figure Caption element represents a caption
    or legend describing the rest of the contents of its parent `<figure>`
    element.
    """

    __slots__ = ()
    tag = "figcaption"


FigCaption = FigureCaption


class Divider(Void):
    """The HTML `<hr>` element represents a thematic break between
    paragraph-level elements: for example, a change of scene in a story, or a
    shift of topic within a section.
    """

    __slots__ = ()
    tag = "hr"


HR = Divider


class OrderedList(Container):
    """The HTML `<ol>` element represents an ordered list of items — typically
    rendered as a numbered list.
    """

    __slots__ = ()
    tag = "ol"


OL = OrderedList


class UnorderedList(Container):
    """The HTML `<ul>` element represents an unordered list of items, typically
    rendered as a bulleted list.
    """

    __slots__ = ()
    tag = "ul"


UL = UnorderedList


class Paragraph(Container):
    """The HTML `<p>` element represents a paragraph."""

    __slots__ = ()
    tag = "p"


P = Paragraph


class PreformattedText(Container):
    """The HTML `<pre>` element represents preformatted text which is to be
    presented exactly as written in the HTML file.
    """

    __slots__ = ()
    tag = "pre"


Pre = PreformattedText



from .base_classes import Container


class Canvas(Container):
    """Use the HTML `<canvas>` element with either the [canvas scripting
    API](https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API) or the
    [WebGL API](https://developer.mozilla.org/en-US/docs/Web/API/WebGL_API) to
    draw graphics and animations.

    You may (and should) provide alternate content inside the `<canvas>`
    block. That content will be rendered both on older browsers that don't
    support canvas and in browsers with JavaScript disabled. Providing a
    useful fallback text or sub DOM helps to make the the canvas more
    accessible.
    """

    __slots__ = ()
    tag = "canvas"


class NoScript(Container):
    """The HTML `<noscript>` element defines a section of HTML to be inserted
    if a script type on the page is unsupported or if scripting is currently
    turned off in the browser.
    """

    __slots__ = ()
    tag = "noscript"


class Script(Container):
    """The HTML `<script>` element is used to embed executable code or data;
    this is typically used to embed or refer to JavaScript code.
    """

    __slots__ = ()
    tag = "script"
